@@ -226,14 +226,28 @@ class Application:
         self.app_state = ApplicationState(app_vals)
 
         # If there are no precompiles, we can build the programs
-        # with what we already have
+        # with what we already have and don't need to pass an
+        # algod client
         if len(self.precompiles) == 0:
             self.compile()
 
-    def compile(self) -> tuple[str, str]:
+    def compile(self, client: Optional[AlgodClient] = None) -> tuple[str, str]:
+        """Fully compile the application to TEAL
+
+        Note: If the application has ``Precompile`` fields, the ``client`` must be passed to
+        compile them into bytecode.
+
+        Args:
+            client (optional): An Algod client that can be passed to ``Precompile`` to have them fully compiled.
+        """
         if self.approval_program is not None and self.clear_program is not None:
             return self.approval_program, self.clear_program
 
+        if len(self.precompiles) > 0:
+            # make sure all the precompiles are available
+            for precompile in self.precompiles.values():
+                precompile.compile(client)
+
         self.router = Router(
             name=self.__class__.__name__,
             bare_calls=BareCallActions(**self.bare_externals),
@@ -308,14 +322,18 @@ class Application:
         return Approve()
 
     def dump(self, directory: str = ".", client: Optional[AlgodClient] = None):
+        """write out the artifacts generated by the application to disk
+
+        Args:
+            directory (optional): str path to the directory where the artifacts should be written
+            client (optional): AlgodClient to be passed to any precompiles
+        """
         if self.approval_program is None:
-            if client is not None:
-                ac = AppPrecompile(self)
-                ac.compile(client)
-            else:
+            if len(self.precompiles) > 0 and client is None:
                 raise Exception(
                     "Approval program empty, if you have precompiles, pass an Algod client to build the precompiles"
                 )
+            self.compile(client)
 
         import json
         import os
