@@ -1,6 +1,6 @@
 from abc import abstractmethod, ABC
 from copy import copy
-from typing import Mapping, cast, Any
+from typing import Mapping, cast, Any, Optional
 from algosdk.future.transaction import StateSchema
 from pyteal import (
     abi,
@@ -20,7 +20,9 @@ from pyteal import (
     Seq,
     If,
 )
+from beaker.lib.storage import LocalBlob
 from beaker.consts import MAX_GLOBAL_STATE, MAX_LOCAL_STATE
+from beaker.lib.storage.global_blob import GlobalBlob
 
 
 def get_default_for_type(stack_type, default):
@@ -43,6 +45,16 @@ def stack_type_to_string(st: TealType):
 
 
 class StateValue(Expr):
+    """Base Class for state values
+
+    Attributes:
+        stack_type: The type of the state value (either TealType.bytes or TealType.uint64)
+        key: key to use to store the the value, default is name of class variable
+        default: Default value for the state value
+        static: Boolean flag to denote that this state value can only be set once and not deleted.
+        descr: Description of the state value to provide some information to clients
+    """
+
     def __init__(
         self,
         stack_type: TealType,
@@ -126,12 +138,26 @@ class StateValue(Expr):
     def get_else(self, val: Expr) -> Expr:
         """gets the value stored at the key. if none is stored, return the value passed"""
 
+    @abstractmethod
+    def exists(self) -> Expr:
+        """checks if the value is set (to whatever value. Returns Int(1) if value is set, Int(0) otherwise."""
+
     @abstractmethod
     def delete(self) -> Expr:
         """deletes the key from state, if the value is static it will be a compile time error"""
 
 
-class DynamicStateValue(ABC):
+class ReservedStateValue(ABC):
+    """Base Class for ReservedStateValues
+
+    Attributes:
+        stack_type (TealType): The type of the state value (either TealType.bytes or TealType.uint64)
+        max_keys (int): Maximum number of keys to reserve for this reserved state value
+        key_gen (subroutine): A subroutine returning TealType.bytes, used to create a key where some data is stored.
+        descr (str): Description of the state value to provide some information to clients
+
+    """
+
     def __init__(
         self,
         stack_type: TealType,
@@ -154,6 +180,16 @@ class DynamicStateValue(ABC):
 
 
 class ApplicationStateValue(StateValue):
+    """Allows storage of state values for an application (global state)
+
+    Attributes:
+        stack_type: The type of the state value (either TealType.bytes or TealType.uint64)
+        key: key to use to store the the value, default is name of class variable
+        default: Default value for the state value
+        static: Boolean flag to denote that this state value can only be set once and not deleted.
+        descr: Description of the state value to provide some information to clients
+    """
+
     def __str__(self) -> str:
         return f"ApplicationStateValue {self.key}"
 
@@ -198,6 +234,9 @@ class ApplicationStateValue(StateValue):
 
         return If((v := App.globalGetEx(Int(0), self.key)).hasValue(), v.value(), val)
 
+    def exists(self) -> Expr:
+        return Seq(val := self.get_maybe(), val.hasValue())
+
     def delete(self) -> Expr:
         check_not_static(self)
 
@@ -207,7 +246,18 @@ class ApplicationStateValue(StateValue):
         return App.globalDel(self.key)
 
 
-class DynamicApplicationStateValue(DynamicStateValue):
+class ReservedApplicationStateValue(ReservedStateValue):
+    """Reserved Application State (global state)
+
+    Used when there should be a number of reserved state fields but the keys are uncertain at build time.
+
+    Attributes:
+        stack_type (TealType): The type of the state value (either TealType.bytes or TealType.uint64)
+        max_keys (int): Maximum number of keys to reserve for this reserved state value
+        key_gen (SubroutineFnWrapper): A subroutine returning TealType.bytes, used to create a key where some data is stored.
+        descr (str): Description of the state value to provide some information to clients
+    """
+
     def __init__(
         self,
         stack_type: TealType,
@@ -238,9 +288,26 @@ class DynamicApplicationStateValue(DynamicStateValue):
 
 
 class AccountStateValue(StateValue):
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-        self.acct = Txn.sender()
+    """Allows storage of keyed values for an account opted into an application (local state)
+
+    Attributes:
+        stack_type: The type of the state value (either TealType.bytes or TealType.uint64)
+        key: key to use to store the the value, default is name of class variable
+        default: Default value for the state value
+        static: Boolean flag to denote that this state value can only be set once and not deleted.
+        descr: Description of the state value to provide some information to clients
+    """
+
+    def __init__(
+        self,
+        stack_type: TealType,
+        key: Expr = None,
+        default: Expr = None,
+        static: bool = False,
+        descr: str = None,
+    ):
+        super().__init__(stack_type, key, default, static, descr)
+        self.acct: Expr = Txn.sender()
 
     def __str__(self) -> str:
         return f"AccountStateValue {self.acct} {self.key}"
@@ -303,6 +370,9 @@ class AccountStateValue(StateValue):
             val,
         )
 
+    def exists(self) -> Expr:
+        return Seq(val := self.get_maybe(), val.hasValue())
+
     def delete(self) -> Expr:
         if self.key is None:
             raise TealInputError(f"AccountStateValue {self} has no key defined")
@@ -317,7 +387,18 @@ class AccountStateValue(StateValue):
         return asv
 
 
-class DynamicAccountStateValue(DynamicStateValue):
+class ReservedAccountStateValue(ReservedStateValue):
+    """Reserved Account State (local state)
+
+    Used when there should be a number of reserved state fields but the keys are uncertain at build time.
+
+    Attributes:
+        stack_type (TealType): The type of the state value (either TealType.bytes or TealType.uint64)
+        max_keys (int): Maximum number of keys to reserve for this reserved state value
+        key_gen (SubroutineFnWrapper): A subroutine returning TealType.bytes, used to create a key where some data is stored.
+        descr (str): Description of the state value to provide some information to clients
+    """
+
     def __init__(
         self,
         stack_type: TealType,
@@ -343,6 +424,115 @@ class DynamicAccountStateValue(DynamicStateValue):
         return AccountStateValue(stack_type=self.stack_type, key=cast(Expr, key))
 
 
+class StateBlob(ABC):
+    def __init__(self, num_keys: int):
+        self.num_keys = num_keys
+
+    @abstractmethod
+    def initialize(self) -> Expr:
+        ...
+
+    @abstractmethod
+    def read(self, start: Expr, stop: Expr) -> Expr:
+        """
+        Reads some bytes from the buffer
+
+        Args:
+            start: An ``Expr`` that represents the start index to read from. Should evaluate to ``uint64``.
+            stop: An ``Expr`` that represents the stop index to read until. Should evaluate to ``uint64``.
+        Returns:
+            The bytes read from the blob from start to stop
+        """
+        ...
+
+    @abstractmethod
+    def write(self, start: Expr, buff: Expr) -> Expr:
+        """
+        Writes the buffer to the blob
+
+        Args:
+            start: An ``Expr`` that represents where to start writing. Should evaluate to ``uint64``.
+            buff: An ``Expr`` that represents the bytes to write. Should evaluate to ``bytes``.
+
+        """
+        ...
+
+    @abstractmethod
+    def read_byte(self, idx: Expr) -> Expr:
+        """
+        Reads a single byte from the given index
+
+        Args:
+            idx: An ``Expr`` that represents the index into the blob to read the byte from. Should evaluate to ``uint64``.
+
+        Returns:
+            A single byte as a ``uint64``
+
+        """
+        ...
+
+    @abstractmethod
+    def write_byte(self, idx: Expr, byte: Expr) -> Expr:
+        """
+        Writes a single byte to the given index
+
+        Args:
+            idx: An ``Expr`` that represents the index to write the byte to. Should evaluate to ``uint64``.
+            byte: An ``Expr`` That represents the index to write the byte to. Should evaluate to ``uint64``.
+
+        """
+        ...
+
+
+class AccountStateBlob(StateBlob):
+    def __init__(self, keys: Optional[int | list[int]] = None):
+        self.blob = LocalBlob(keys=keys)
+        self.acct: Expr = Txn.sender()
+
+        super().__init__(self.blob._max_keys)
+
+    def __getitem__(self, acct: Expr) -> "AccountStateBlob":
+        asv = copy(self)
+        asv.acct = acct
+        return asv
+
+    def initialize(self) -> Expr:
+        return self.blob.zero(acct=self.acct)
+
+    def write(self, start: Expr, buff: Expr) -> Expr:
+        return self.blob.write(start, buff, acct=self.acct)
+
+    def read(self, start: Expr, stop: Expr) -> Expr:
+        return self.blob.read(start, stop, acct=self.acct)
+
+    def read_byte(self, idx: Expr) -> Expr:
+        return self.blob.get_byte(idx, acct=self.acct)
+
+    def write_byte(self, idx: Expr, byte: Expr) -> Expr:
+        return self.blob.set_byte(idx, byte, acct=self.acct)
+
+
+class ApplicationStateBlob(StateBlob):
+    def __init__(self, keys: Optional[int | list[int]] = None):
+        self.blob = GlobalBlob(keys=keys)
+        super().__init__(self.blob._max_keys)
+
+    def initialize(self) -> Expr:
+        return self.blob.zero()
+
+    def write(self, start: Expr, buff: Expr) -> Expr:
+        return self.blob.write(start, buff)
+
+    def read(self, start: Expr, stop: Expr) -> Expr:
+        return self.blob.read(start, stop)
+
+    def read_byte(self, idx: Expr) -> Expr:
+        return self.blob.get_byte(idx)
+
+    def write_byte(self, idx: Expr, byte: Expr) -> Expr:
+        return self.blob.set_byte(idx, byte)
+
+
 def check_not_static(sv: StateValue):
     if sv.static:
         raise TealInputError(f"StateValue {sv} is static")
@@ -360,38 +550,52 @@ def check_match_type(sv: StateValue, val: Expr):
 
 
 class State:
-    """holds all the declared and dynamic state values for this storage type"""
+    """holds all the declared and reserved state values for this storage type"""
 
-    def __init__(self, fields: Mapping[str, StateValue | DynamicStateValue]):
+    def __init__(
+        self, fields: Mapping[str, StateValue | ReservedStateValue | StateBlob]
+    ):
         self.declared_vals: dict[str, StateValue] = {
             k: v for k, v in fields.items() if isinstance(v, StateValue)
         }
-
         self.__dict__.update(self.declared_vals)
 
-        self.dynamic_vals: dict[str, DynamicStateValue] = {
-            k: v for k, v in fields.items() if isinstance(v, DynamicStateValue)
+        self.blob_vals: dict[str, StateBlob] = {
+            k: v for k, v in fields.items() if isinstance(v, StateBlob)
         }
-        self.__dict__.update(self.dynamic_vals)
+        self.__dict__.update(self.blob_vals)
+
+        self.reserved_vals: dict[str, ReservedStateValue] = {
+            k: v for k, v in fields.items() if isinstance(v, ReservedStateValue)
+        }
+        self.__dict__.update(self.reserved_vals)
 
         self.num_uints = len(
             [l for l in self.declared_vals.values() if l.stack_type == TealType.uint64]
         ) + sum(
             [
                 l.max_keys
-                for l in self.dynamic_vals.values()
+                for l in self.reserved_vals.values()
                 if l.stack_type == TealType.uint64
             ]
         )
 
-        self.num_byte_slices = len(
-            [l for l in self.declared_vals.values() if l.stack_type == TealType.bytes]
-        ) + sum(
-            [
-                l.max_keys
-                for l in self.dynamic_vals.values()
-                if l.stack_type == TealType.bytes
-            ]
+        self.num_byte_slices = (
+            len(
+                [
+                    l
+                    for l in self.declared_vals.values()
+                    if l.stack_type == TealType.bytes
+                ]
+            )
+            + sum(
+                [
+                    l.max_keys
+                    for l in self.reserved_vals.values()
+                    if l.stack_type == TealType.bytes
+                ]
+            )
+            + sum([b.num_keys for b in self.blob_vals.values()])
         )
 
     def dictify(self) -> dict[str, dict[str, Any]]:
@@ -401,30 +605,20 @@ class State:
                 k: {
                     "type": stack_type_to_string(v.stack_type),
                     "key": v.str_key(),
-                    "descr": v.descr,
+                    "descr": v.descr if v.descr is not None else "",
                 }
                 for k, v in self.declared_vals.items()
             },
-            "dynamic": {
+            "reserved": {
                 k: {
                     "type": stack_type_to_string(v.stack_type),
-                    "max-keys": v.max_keys,
-                    "descr": v.descr,
+                    "max_keys": v.max_keys,
+                    "descr": v.descr if v.descr is not None else "",
                 }
-                for k, v in self.dynamic_vals.items()
+                for k, v in self.reserved_vals.items()
             },
         }
 
-    def initialize(self) -> Expr:
-        """Generate expression from state values to initialize a default value"""
-        return Seq(
-            *[
-                v.set_default()
-                for v in self.declared_vals.values()
-                if not v.static or (v.static and v.default is not None)
-            ]
-        )
-
     def schema(self) -> StateSchema:
         """gets the schema as num uints/bytes for app create transactions"""
         return StateSchema(
@@ -435,7 +629,12 @@ class State:
 class ApplicationState(State):
     def __init__(
         self,
-        fields: Mapping[str, ApplicationStateValue | DynamicApplicationStateValue] = {},
+        fields: Mapping[
+            str,
+            ApplicationStateValue
+            | ReservedApplicationStateValue
+            | ApplicationStateBlob,
+        ],
     ):
         super().__init__(fields)
         if (total := self.num_uints + self.num_byte_slices) > MAX_GLOBAL_STATE:
@@ -443,10 +642,24 @@ class ApplicationState(State):
                 f"Too much application state, expected {total} <= {MAX_GLOBAL_STATE}"
             )
 
+    def initialize(self) -> Expr:
+        """Generate expression from state values to initialize a default value"""
+        return Seq(
+            *[
+                v.set_default()
+                for v in self.declared_vals.values()
+                if not v.static or (v.static and v.default is not None)
+            ]
+            + [v.initialize() for v in self.blob_vals.values()]
+        )
+
 
 class AccountState(State):
     def __init__(
-        self, fields: Mapping[str, AccountStateValue | DynamicAccountStateValue]
+        self,
+        fields: Mapping[
+            str, AccountStateValue | ReservedAccountStateValue | AccountStateBlob
+        ],
     ):
         super().__init__(fields)
         if (total := self.num_uints + self.num_byte_slices) > MAX_LOCAL_STATE:
@@ -462,4 +675,5 @@ class AccountState(State):
                 for v in self.declared_vals.values()
                 if not v.static or (v.static and v.default is not None)
             ]
+            + [v.initialize() for v in self.blob_vals.values()]
         )
